use std::ops::RangeInclusive;

use rand::{distributions::Uniform, Rng as _, SeedableRng};
use rayon::iter::{IntoParallelIterator, ParallelIterator};
use sort_machine::{Instruction, Machine, Value};

use crate::{Rng, CORE_COUNT};

const TESTCASE_COUNT: u64 = 64;
const SEED_BASE: u64 = 0x3a0daf0066817f25;
const RANGE: RangeInclusive<Value> = -1_000_000_000_000..=1_000_000_000_000;

fn judge_single(seed: u64, program: &[Instruction]) -> bool {
    let rng = Rng::seed_from_u64(seed);
    let uniform = Uniform::new_inclusive(RANGE.start(), RANGE.end());

    let input: Box<[Value]> = rng.sample_iter(uniform).take(CORE_COUNT).collect();

    let expected = input.iter().sum();

    let mut machine = Machine::new(CORE_COUNT, Box::new([input]));

    for inst in program {
        machine.exec(*inst);
    }

    machine.output()[0] == expected
}

pub fn judge(program: Vec<Instruction>) -> bool {
    (0..TESTCASE_COUNT)
        .into_par_iter()
        .all(|i| judge_single(SEED_BASE ^ i, &program))
}
