#!/bin/bash

echo "Compiling judge:"

(cd judge && cargo build --release)

judge="./judge/target/release/sort-machine-home-judge"

check_solution () {
    task="$1"
    solution="$2"
    expected="$3"

    task_name=$(basename "$task")
    solution_name=$(basename "$solution")

    output=$(mktemp)
    score_file=$(mktemp)
    message_file=$(mktemp)

    (cd $(dirname "$solution") && "./$solution_name") > "$output"
    $judge "inputs/$task_name.in" "/dev/null" "$output" > "$score_file" 2> "$message_file"

    score=$(cat "$score_file")
    message=$(cat "$message_file")

    rm "$output" "$score_file" "$message_file"

    if [ "$score" == "$expected" ]; then
        validity="OK"
    else
        validity="ERROR"
    fi

    echo "${solution_name}: ${message} (${validity})"
}

for task_dir in solutions/*; do
    task_name=$(basename "$task_dir")
    echo "Testing task ${task_name}:"

    for solution in "$task_dir"/correct/*; do
        [[ -e "$solution" ]] || break;
        check_solution "$task_dir" "$solution" "1" 
    done

    for solution in "$task_dir"/wrong/*; do
        [[ -e "$solution" ]] || break;
        check_solution "$task_dir" "$solution" "0" 
    done
done
