use std::{
    io::{self, stdout},
    iter,
};

use csv::{ErrorKind, Writer};
use sort_machine::Machine;

fn print_csv_inner(machine: &Machine, insts: &[String]) -> Result<(), csv::Error> {
    let stdout = stdout().lock();
    let mut writer = Writer::from_writer(stdout);

    let mut buf = itoa::Buffer::new();

    writer.write_field("number")?;
    writer.write_field("instruction")?;

    for i in 0..machine.cores() {
        writer.write_field(buf.format(i))?;
    }

    writer.write_record(iter::empty::<&[u8]>())?;

    for (i, input) in machine.inputs().iter().enumerate() {
        let i = i + 1;

        writer.write_field("")?;
        writer.write_field(format!("input {i}"))?;

        for i in input.iter() {
            writer.write_field(buf.format(*i))?;
        }

        writer.write_record(iter::empty::<&[u8]>())?;
    }

    for (i, (result, inst)) in machine.results().iter().zip(insts.iter()).enumerate() {
        let i = i + 1;

        writer.write_field(buf.format(i))?;
        writer.write_field(inst)?;

        for i in result.iter() {
            writer.write_field(buf.format(*i))?;
        }

        writer.write_record(iter::empty::<&[u8]>())?;
    }

    writer.flush()?;

    Ok(())
}

pub fn print_csv(machine: &Machine, insts: &[String]) -> Result<(), io::Error> {
    print_csv_inner(machine, insts).map_err(|error| match error.into_kind() {
        ErrorKind::Io(error) => error,
        error => panic!("impossible error when serializing CSV: {error:?}"),
    })
}
