use std::num::Saturating;

use sort_machine::Value;

use crate::simulation::{Action, ParseOutput, SimulationError};

const MAX_MEMORY_FOR_RUN: usize = 1024 * 1024 * 1024;
const MAX_MEMORY_FOR_TRACE: usize = 8 * 256 * 512;

const MAX_TOTAL_MEMORY_USAGE: usize = 3 * 1024 * 1024 * 1024;
const BYTES_PER_PERMIT: usize = 1024 * 1024;
pub const INITIAL_PERMITS: usize = MAX_TOTAL_MEMORY_USAGE / BYTES_PER_PERMIT;

const MAX_CORES: usize = 1024 * 1024;

impl ParseOutput {
    pub fn estimate_memory_usage(&self) -> usize {
        let cores = Saturating(self.machine.cores());
        let inputs = Saturating(self.machine.inputs().len());
        let instructions = Saturating(self.program.len());
        let value_size = Saturating(size_of::<Value>());

        let memory = cores * (inputs + instructions) * value_size;
        memory.0
    }

    pub fn permits_needed(&self) -> u32 {
        u32::try_from(self.estimate_memory_usage() / BYTES_PER_PERMIT).unwrap_or(u32::MAX)
    }

    pub fn check_limits(&self, action: Action) -> Result<(), SimulationError> {
        if self.machine.cores() > MAX_CORES {
            return Err(SimulationError::CoreLimit { cores: MAX_CORES });
        }

        let limit = match action {
            Action::Run => MAX_MEMORY_FOR_RUN,
            Action::Trace => MAX_MEMORY_FOR_TRACE,
        };

        if self.estimate_memory_usage() <= limit {
            Ok(())
        } else {
            Err(SimulationError::MemoryLimit {
                action,
                mebibytes: limit / (1024 * 1024),
            })
        }
    }
}
